import java.io.*;
import java.util.ArrayList;

import javax.vecmath.Point2d;

public class PolylineReader {
	/**
	 * A 2d-polyline-array from a given file.
	 * <pre>Fileformate:
	 * * one point per line
	 * * double entrys in each component
	 * * entrys are separated by whitespaces</pre>
	 * @param filename name of the file
	 * @throws RuntimeException on invalid file format
	 * @throws RuntimeException on file not found
	 * @throws RuntimeException on I/O Error
	 * @throws RuntimeException on point parse error
	 * @return the 2d polyline as an array of points
	 */
	public static Point2d[] readPolyline2Array(String filename) {
		ArrayList<Point2d> polyline = readPolyline(filename);
		if(polyline == null)
			return null;
		return polyline.toArray(new Point2d[0]);
	}
	
	/**
	 * A 2d-polyline-arraylist from a given file.
	 * <pre>Fileformate:
	 * * one point per line
	 * * double entrys in each component
	 * * entrys are separated by whitespaces</pre>
	 * @param filename name of the file
	 * @throws RuntimeException on invalid file format
	 * @throws RuntimeException on file not found
	 * @throws RuntimeException on I/O Error
	 * @throws RuntimeException on point parse error
	 * @return the 2d polyline as an arraylist of points
	 */
	public static ArrayList<Point2d> readPolyline(String filename) {
		ArrayList<Point2d> polyline = new ArrayList<Point2d>();
		
		int lineIdx = 0;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(
					new File(filename)));
			String line;
			
			// for each line of the file
			while((line = reader.readLine()) != null) {
				// splitt at white spaces (ignore multible whitespaces)
				// \s is the whitespace pattern
				// see: {@link java.util.regex.Pattern}
				String[] pointStr = line.split("\\s+");
				if(pointStr.length != 2)
					throw new RuntimeException("Invalid file format in line " + (lineIdx + 1) + "!");

				// add the point to the polyline
				polyline.add(new Point2d(
						Double.parseDouble(pointStr[0]),
						Double.parseDouble(pointStr[1])));
				
				lineIdx++;
			}
			// file successful readed
		} 
		// the catches are used to "replace" all non-RuntimeExceptions with RuntimeExceptions
		catch (FileNotFoundException e) {
			throw new RuntimeException("File not found!", e);
		} catch (IOException e) {
			throw new RuntimeException("I/O error", e);
		} catch (NumberFormatException e) {
			throw new RuntimeException("Point parse error in line " + lineIdx + "!", e);
		}
		
		return polyline;
	}
}
